#include <stdlib.h>
#include <stdio.h>
#include "coord.h"

coord_list* newCoordList() {
	coord_list* new;
	int i = 0;

        new=(coord_list*)malloc(sizeof(coord_list));

	new->cnt = 0;

	for(i=0;i<MAX_MONSTERS;i++) {
		new->node[i].who = NULL;
		new->node[i].x   = -1;
		new->node[i].y   = -1;
	}

	return new;
}

void zapCoordList(coord_list* cl) {
	free(cl);
	cl = NULL;
}

void addCoordMonster(coord_list* cl, monster* who, int x, int y) {
	if(cl->cnt < (MAX_MONSTERS-1)) {
		cl->node[cl->cnt].who = who;
		cl->node[cl->cnt].x   = x;
		cl->node[cl->cnt].y   = y;
		cl->cnt++;
	} else {
		if(VERBOSE_COORD_ERRORS)
			fprintf(stderr,"addCoordMonster: coord list is full\n");
	}
}

void delCoordMonster(coord_list* cl, monster* who) {
	int i = 0;

	if(cl->cnt > 0) {
		while(cl->node[i].who != who) i++;

		cl->node[i].who = NULL;
		cl->node[i].x   = -1;
		cl->node[i].y   = -1;

		i++;
		
		while(i <= cl->cnt) {
			cl->node[i-1].who = cl->node[i].who;
			cl->node[i-1].x   = cl->node[i].x;
			cl->node[i-1].y   = cl->node[i].y;
			i++;
		}
		
		cl->cnt--;
	} else {
		if(VERBOSE_COORD_ERRORS)
			fprintf(stderr,"delCoordMonster: coord list is empty\n");
	}
}

int getX(coord_list* cl, monster* who) {
	int i = 0;

	while(cl->node[i].who != who) i++;

	return cl->node[i].x;
}

int getY(coord_list* cl, monster* who) {
	int i = 0;

	while(cl->node[i].who != who) i++;

	return cl->node[i].y;
}

monster* getCoordMonster(coord_list* cl, int x, int y) {
	int i = 0;
	
	do {
		if(cl->node[i].x==x && cl->node[i].y==y)
			return cl->node[i].who;
		i++;
	} while(i <= cl->cnt);

	return NULL;
}

void moveCoord(coord_list* cl, int x, int y, int dx, int dy, int teleport) {
	monster* who;
	int i = 0;

	if(!getCoordMonster(cl,x+dx,y+dy)) {
		if(who=getCoordMonster(cl,x,y)) {
			while(cl->node[i].who != who) i++;

			if((teleport) || (abs(dx)<=1 && abs(dy)<=1)) {
				cl->node[i].x=x+dx;
				cl->node[i].y=y+dy;
			} else {
				if(VERBOSE_COORD_ERRORS) 
					fprintf(stderr,"moveCoord: monster can't move that far without teleporting\n");
			}
		} else {
			if(VERBOSE_COORD_ERRORS)
				fprintf(stderr,"moveCoord: no monster to move\n");
		}
	} else {
		if(VERBOSE_COORD_ERRORS) 
			fprintf(stderr,"moveCoord: a monster is in the way\n");
	}
}

void moveCoordMonster(coord_list* cl, monster* who, int dx, int dy, int teleport) {
	moveCoord(cl,getX(cl,who),getY(cl,who),dx,dy,teleport);
}
