#include <stdio.h>
#include <math.h>
#include "lists.h"


void addMonster(turn_list *tl, coord_list *cl, map *map, monster* who, int x, int y) {
	addTurnMonster(tl,who);
	addCoordMonster(cl,who,x,y);
	addMapMonster(map,who,x,y);
}

void delMonster(turn_list *tl, coord_list *cl, map *map, monster* who) {
	int x,y;

	delTurnMonster(tl,who);
	
	x = getX(cl,who); 
	y = getY(cl,who);
	delCoordMonster(cl,who);

	delMapMonster(map,x,y);
}

void moveMonster(coord_list *cl, map *map, monster* who, int dx, int dy, int teleport) {
	int i = 0,
	    x = getX(cl,who),
	    y = getY(cl,who);

	if(x+dx<MAX_MAP_X && x+dx>=0 && y+dy<MAX_MAP_Y && y+dy>=0) {
		if(who->access & map->cell[x+dx][y+dy]->access) { 
		/* in other words, if who has one of cell's access types*/
			moveMapMonster(map,x,y,dx,dy,teleport);
			moveCoordMonster(cl,who,dx,dy,teleport);
		}
		addUpdate(map,x,y);
		addUpdate(map,x+dx,y+dy);
	} else {
		if(VERBOSE_LIST_ERRORS) 
			fprintf(stderr,"exit map at (%d,%d)\n",x+dx,y+dy);
		/* exitMap(map,x+dx,y+dy); */
	}
}

void delAllMonsters(turn_list *tl, coord_list *cl, map *map) {
	int x,y;

	while(tl->cnt) delTurnMonster(tl,tl->turn[0]);
	while(cl->cnt) {
		x = cl->node[0].x; 
		y = cl->node[0].y;
		delCoordMonster(cl,cl->node[0].who);
		delMapMonster(map,x,y);
	}
}

void relMoveMonster(coord_list *cl, map *map, monster* who, int ox, int oy, int tox, int toy) {
	int movex,
	    movey,
	    dx = (tox-ox),
	    dy = (toy-oy);


	if(dx==0 && dy==0) { 
		movex=0; 
		movey=0; 
	} else if(dx==0 && dy!=0) { 
		if(dy>0) { 
			movex=0; 
			movey=1; 
		} else { 
			movex=0; 
			movey=-1; 
		}
	} else if(dx!=0 && dy==0) { 
		if(dx>0) { 
			movex=1; 
			movey=0; 
		} else {
			movex=-1;
			movey=0; 
		}
	} else {		/* here's the interesting part...
				   I have to get this right to make the monsters smart */ 
		if(dx>0) {
			if(dy>0) {
				if(dx > dy) {
					movex=1;
					movey=0;
				} else if(dx < dy) {
					movex=0;
					movey=1;
				} else {
					movex=1;
					movey=1;
				}
			} else {
				if(dx > -dy) {
					movex=1;
					movey=0;
				} else if(dx < -dy) {
					movex=0;
					movey=-1;
				} else {
					movex=1;
					movey=-1;
				}
			}
		} else {
			if(dy>0) {
				if(-dx > dy) {
					movex=-1;
					movey=0;
				} else if(-dx < dy) {
					movex=0;
					movey=1;
				} else {
					movex=-1;
					movey=1;
				}
			} else {
				if(-dx > -dy) {
					movex=-1;
					movey=0;
				} else if(-dx < -dy) {
					movex=0;
					movey=-1;
				} else {
					movey=-1;
					movex=-1;
				}
			}
		}
	} 
	moveMonster(cl,map,who,movex,movey,0);
}

monster* nearestMonster(coord_list *cl, map *map, monster* who, int friendfoe) {
	monster* gotcha = NULL;
	int i = 0,
	    whox, whoy,
	    dx, dy,
	    bdx, bdy=-1;

	whox = getX(cl,who);
	whoy = getY(cl,who);

	while(i < cl->cnt) {
		if(!(whox == cl->node[i].x && whoy == cl->node[i].y)) {
			if(friendfoe==0 || (friendfoe==1 && cl->node[i].who->is_player==0)
			                || (friendfoe==2 && cl->node[i].who->is_player==1)) {
				dx = abs(cl->node[i].x - whox);
				dy = abs(cl->node[i].y - whoy);
				if(bdy==-1 || ((dx*dx)+(dy*dy))<((bdx*bdx)+(bdy*bdy))) {
					bdx = dx;
					bdy = dy;
					gotcha = cl->node[i].who;
				}
			}
		}
		i++;
	}
	return gotcha;
}

void moveToMonster(coord_list *cl, map *map, monster* who, monster* to) {
	relMoveMonster(cl,map,who,getX(cl,who),getY(cl,who),getX(cl,to),getY(cl,to));
}

void moveToNearestMonster(coord_list *cl, map *map, monster* who, int friendfoe) {
	monster* nearest;

	if(nearest = nearestMonster(cl,map,who,friendfoe)) {
		moveToMonster(cl,map,who,nearest);
	} else {
		if(VERBOSE_LIST_ERRORS)
			fprintf(stderr,"moveToNearestMonster: no monster to move to (friendfoe = %d)\n",friendfoe);
	}
}


void centerAreaOnMonster(coord_list* cl, monster* who) {
	centerArea(getX(cl,who),getY(cl,who));
}
