#include <stdlib.h>
#include <stdio.h>
#include <math.h> 	/* for abs() */
#include <time.h>	/* for time(NULL) used in srand() */
#include "map.h"

map* newMap(char* name) {
	map* new;
	int i,j;

	new=(map*)malloc(sizeof(map));

	setMapName(new,name);

	for(j=0;j<MAX_MAP_Y;j++)
	for(i=0;i<MAX_MAP_X;i++) {
		new->cell[i][j] = (space*)malloc(sizeof(space));

		new->cell[i][j]->type   = 0;
		new->cell[i][j]->status = 0;
		new->cell[i][j]->access = 0;
		new->cell[i][j]->mos    = NULL;
	}
	
	new->up_cnt = 0;

	for(i=0;i<MAX_MAP_UPDATE;i++) {
		new->update[i].x = -1;
		new->update[i].y = -1;
	}

	new->AREA_OX = 0;
	new->AREA_OY = 0;

	initMap(new);
	
	return new;
}

void initMap(map* new) {
	int i,j;

	srand(time(NULL));

	for(j=0;j<MAX_MAP_Y;j++)
	for(i=0;i<MAX_MAP_X;i++) {
		new->cell[i][j]->type   = (rand()%8)?grass:pine;
		new->cell[i][j]->status = rand()%2;
		
		switch(new->cell[i][j]->type) {
			case grass : new->cell[i][j]->access = ethereal | fly | walk; break;
			case pine  : new->cell[i][j]->access = ethereal | fly; break;

			default	: new->cell[i][j]->access = ethereal; break;
		}
	}
	
}

void zapMap(map* map) {
	int i,j;

	for(j=0;j<MAX_MAP_Y;j++)
	for(i=0;i<MAX_MAP_X;i++) {
		if(map->cell[i][j]->mos) {
			free(map->cell[i][j]->mos);
			map->cell[i][j] = NULL;
		}
		free(map->cell[i][j]);
		map->cell[i][j] = NULL;
	}
	
	free(map);
	map = NULL;
}

void setMapName(map* map, char* name) {
        strcpy(map->name,name);
}

char* getMapName(map* map) {
        return map->name;
}

void addUpdate(map* map, int x, int y) {
	int already_there = 0,
	    i = 0;

	while(i < map->up_cnt) {
		if(map->update[i].x == x && map->update[i].y == y)
			already_there = 1;
		i++;
	}

	if(!already_there) {
		if(map->up_cnt < (MAX_MAP_UPDATE-1)) {
			map->update[map->up_cnt].x = x;
			map->update[map->up_cnt].y = y;
			map->up_cnt++;
		} else {
			if(VERBOSE_MAP_ERRORS) 
				fprintf(stderr,"addUpdate: update stack is full\n");
		}
	} else {
		if(VERBOSE_MAP_ERRORS) 
			fprintf(stderr,"addUpdate: space would already be updated\n");
	}
}

void delUpdate(map* map) {
	if(map->up_cnt > 0) {
		map->update[map->up_cnt-1].x = -1;
		map->update[map->up_cnt-1].y = -1;
		map->up_cnt--;
	} else {
		if(VERBOSE_MAP_ERRORS) 
			fprintf(stderr,"delUpdate: update stack is empty\n");
	}
}

void setSpaceType(map* map, int x, int y, int type) {
	map->cell[x][y]->type = type;
}

int getSpaceType(map* map, int x, int y) {
	return map->cell[x][y]->type;
}

void setMos(map* map, monster* who, int x, int y) {
	map->cell[x][y]->mos = who;
}

monster* getMos(map* map, int x, int y) {
	return map->cell[x][y]->mos;
}

void addMapMonster(map* map, monster* who, int x, int y) {
	if(!getMos(map,x,y)) {
		setMos(map,who,x,y);
	} else {
		if(VERBOSE_MAP_ERRORS) 
			fprintf(stderr,"addMapMonster: that space is taken\n");
	}
}

void delMapMonster(map* map, int x, int y) {
	if(getMos(map,x,y)) {
		setMos(map,NULL,x,y);
	} else {
		if(VERBOSE_MAP_ERRORS) 
			fprintf(stderr,"delMapMonster: there is no monster to delete\n");
	}
}

void moveMapMonster(map* map, int x, int y, int dx, int dy, int teleport) {
	monster* who=getMos(map,x,y);

	if(who) {
		if((teleport) || (abs(dx)<=1 && abs(dy)<=1)) {
			if(!getMos(map,x+dx,y+dy)) {
				delMapMonster(map,x,y);
				addMapMonster(map,who,x+dx,y+dy);
			} else {
				if(VERBOSE_MAP_ERRORS)
					fprintf(stderr,"moveMapMonster: space (%d,%d) taken\n",x+dx,y+dy);
			}
		} else {
			if(VERBOSE_MAP_ERRORS)
				fprintf(stderr,"moveMapMonster: can't move (%+d,%+d) without teleporting\n",dx,dy);
		}
	} else {
		if(VERBOSE_MAP_ERRORS) 
			fprintf(stderr,"moveMapMonster: no monster on space (%d,%d) to move\n",x,y);
	}
}

char* getMapMonsterName(map *map, int x, int y) {
	monster* who=getMos(map,x,y);

	if(who) return who->name;
	return NULL;
}
