#ifndef MAP_H
#define MAP_H

#include <stdlib.h>
#include "monster.h" 	/* for the monster mos field */

#define VERBOSE_MAP_ERRORS	(0)

#define MAX_MAP_X 		(32)
#define MAX_MAP_Y 		(32)


#define MAX_MAP_UPDATE		(512) 
/* make sure this is greater than (XSPACES*YSPACES) !!! */

typedef struct update_stack {
	int x;
	int y;
} update_stack;

typedef struct space {
	int      type;
	int      status;
	int      access;
	monster* mos;
} space;

typedef struct map {
	char		name[32];
	space*		cell[MAX_MAP_X][MAX_MAP_Y];
	int		up_cnt;
	update_stack	update[MAX_MAP_UPDATE];

	int		AREA_OX;
	int		AREA_OY;
} map;


enum space_type {
	empty,		/* 0, no terrain (hacky surrealistic terrain) */
	grass,		/* 1 */
	pine,		/* 2 */
	rock		/* 3 */
};


/* the access types for monsters and spaces */
#define forbidden	0
#define ethereal	1
#define fly		2
#define swim		4
#define walk		8

map* newMap(char* name);
void initMap(map* map);
void zapMap(map* map);

void  setMapName(map* map, char* name);
char* getMapName(map* map);

void addUpdate(map* map, int x, int y);
void delUpdate(map* map);

void setSpaceType(map* map, int x, int y, int type);
int  getSpaceType(map* map, int x, int y);

void     setMos(map* map, monster* who, int x, int y);
monster* getMos(map* map, int x, int y);

void addMapMonster(map* map, monster* who, int x, int y);
void delMapMonster(map* map, int x, int y);

void moveMapMonster(map* map, int x, int y, int dx, int dy, 
                    int teleport);

char* getMapMonsterName(map *map, int x, int y);


#endif /* MAP_H */
