#include <stdlib.h>
#include "turn.h"

#include "lists.h"	/* for the lists functions */
#include "area.h"	/* for the refresh functions */

turn_list* newTurnList() {
	turn_list* new;
	int i = 0;

	new=(turn_list*)malloc(sizeof(turn_list));

	new->cnt = 0;
	new->cur = -1;
	new->game_control = no_action;

	for(i=0;i<MAX_MONSTERS;i++) {
		new->turn[i] = NULL;
	}

	return new;
}

void zapTurnList(turn_list* tl) {
	free(tl);
	tl = NULL;
}

void addTurnMonster(turn_list* tl, monster* who) {
	if(tl->cnt < (MAX_MONSTERS-1)) {
		tl->cnt++;
		tl->turn[tl->cnt-1] = who;
		if(tl->cur == -1) tl->cur = 0;
	} else {
		if(VERBOSE_TURN_ERRORS)
			fprintf(stderr,"addTurnMonster: turn list is full\n");
	}
}

monster* whoTurn(turn_list* tl) {
	if(tl->cnt > 0) {
		return tl->turn[tl->cur];
	} else {
		if(VERBOSE_TURN_ERRORS)
			fprintf(stderr,"whoTurn: nobody in the turn list\n");
	}
	
	return NULL;
}

/* still have to build in a check if the monster exists */
void delTurnMonster(turn_list* tl, monster* who) {
	int i = 0;

	if(tl->cnt > 0) {
		while(tl->turn[i] != who) i++;

		tl->turn[i] = NULL;
		i++;

		while(i <= tl->cnt) {
			tl->turn[i-1] = tl->turn[i];
			i++;
		}
		tl->turn[tl->cnt] = NULL;
		tl->cnt--;
	} else {
		if(VERBOSE_TURN_ERRORS)
			fprintf(stderr,"delTurnMonster: turn list is empty\n");
	}
}

void nextTurn(turn_list* tl) {
	if(tl->cnt > 0) {
		if(++tl->cur == tl->cnt) tl->cur = 0;
	} else {
		if(VERBOSE_TURN_ERRORS)
			fprintf(stderr,"nextTurn: nobody in the turn list\n");
	}
}

void doTurn(turn_list* tl, coord_list* cl, map* map) {
	monster* who = whoTurn(tl);
	int i = 0;

	decideGameControl(tl);

	switch(tl->game_control) {
		case pc : 
			if(PLAYER_HIGHLIGHTING) {
				activeRefresh(getX(cl,who),getY(cl,who));
				PasteBufferToWindow();
			}
			centerArea(map,getX(cl,who),getY(cl,who));
			SmartRefresh(map);
			PasteBufferToWindow();
			listen(tl,cl,map,who);

			break;

		case npc :
			if(NPC_REFRESH) {
			SmartRefresh(map);
			PasteBufferToWindow();
        		}

			todo(tl,cl,map,who); 

			break;

		case quit : 
			break;
	}
	nextTurn(tl);
}

void decideGameControl(turn_list* tl) {

	int i = 0;

	if(tl->game_control != quit) {
		if(tl->cnt > 0) {
			if(tl->turn[tl->cur]->is_player) {
				tl->game_control = pc;
			} else {
				tl->game_control = npc;
			}
		} else {
			tl->game_control = no_action;
		} 
	}
}

void dealTurn(turn_list* tl, coord_list* cl, map* map) {
	int a = tl->cur;

	if(tl->cnt > 0) {

		do {
			doTurn(tl,cl,map);
		} while((tl->cur != a) && (tl->game_control != quit));
	}
}
